# -*- coding: utf-8 -*-
"""Module for parsing Nagios 'Extra opts' files."""

import os
import sys

import pynag.Utils

# TODO: Raise more specific errors in this class
from pynag.Parsers.errors import ParserError


_sentinel = object()


class ExtraOptsParser(object):

    """ Get Nagios Extra-Opts from a config file as specified by http://nagiosplugins.org/extra-opts

    We could ALMOST use pythons ConfParser but nagios plugin team thought it would be a
    good idea to support multiple values per key, so a dict datatype no longer works.

    Its a shame because we have to make our own "ini" parser as a result

    Usage::

        # cat /etc/nagios/plugins.ini
        [main]
        host_name = localhost
        [other section]
        host_name = example.com
        # EOF

        e = ExtraOptsParser(section_name='main', config_file='/etc/nagios/plugins.ini')
        e.get('host_name')  # returns "localhost"
        e.get_values()  # Returns a dict of all the extra opts
        e.getlist('host_name')  # returns all values of host_name (if more than one were specified) in a list

    """
    standard_locations = [
        "/etc/nagios/plugins.ini",
        "/usr/local/nagios/etc/plugins.ini",
        "/usr/local/etc/nagios/plugins.ini",
        "/etc/opt/nagios/plugins.ini",
        "/etc/nagios-plugins.ini",
        "/usr/local/etc/nagios-plugins.ini",
        "/etc/opt/nagios-plugins.ini",
    ]

    def __init__(self, section_name=None, config_file=None):
        if not section_name:
            section_name = self.get_default_section_name()
        if not config_file:
            config_file = self.get_default_config_file()
        self.section_name = section_name
        self.config_file = config_file
        self._all_options = self.parse_file(filename=config_file) or {}

    def get_values(self):
        """ Returns a dict with all extra-options with the granted section_name and config_file

        Results are in the form of::

            {
              'key': ["possible","values"]
            }
        """
        return self._all_options.get(self.section_name, {})

    def get_default_section_name(self):
        """ According to extra-opts standard, the default should be filename of check script being run """
        return os.path.basename(sys.argv[0])

    def get_default_config_file(self):
        """ Return path to first readable extra-opt config-file found

        According to the nagiosplugins extra-opts spec the search method is as follows:

            1. Search for nagios.ini or nagios-plugins.ini in : splitted variable NAGIOS_CONFIG_PATH
            2. Search in a predefined list of files
            3. Return None if no config file is found

        The method works as follows:

        To quote the spec on NAGIOS_CONFIG_PATH:

            *"To use a custom location, set a NAGIOS_CONFIG_PATH environment
            variable to the set of directories that should be checked (this is a
            colon-separated list just like PATH). The first plugins.ini or
            nagios-plugins.ini file found in these directories will be used."*

        """
        search_path = []
        nagios_config_path = os.environ.get('NAGIOS_CONFIG_PATH', '')
        for path in nagios_config_path.split(':'):
            search_path.append(os.path.join(path, 'plugins.ini'))
            search_path.append(os.path.join(path, 'nagios-plugins.ini'))

        search_path += self.standard_locations
        self.search_path = search_path
        for path in search_path:
            if os.path.isfile(path):
                return path
        return None

    def get(self, option_name, default=_sentinel):
        """ Return the value of one specific option

        Args:

            option_name: The value set to this option will be returned

        Returns:

            The value of `option_name`

        Raises:

            :py:class:`ValueError` when `option_name` cannot be found in options

        """
        result = self.getlist(option_name, default)

        # If option was not found, raise error
        if result == _sentinel:
            raise ValueError("Option named %s was not found" % (option_name))
        elif result == default:
            return result
        elif not result:
            # empty list
            return result
        else:
            return result[0]

    def getlist(self, option_name, default=_sentinel):
        """ Return a list of all values for option_name

        Args:

            option_name: All the values set to this option will be returned

        Returns:

            List containing all the options set to `option_name`

        Raises:

            :py:class:`ValueError` when `option_name` cannot be found in options

        """
        result = self.get_values().get(option_name, default)
        if result == _sentinel:
            raise ValueError("Option named %s was not found" % (option_name))
        return result

    def parse_file(self, filename):
        """ Parses an ini-file and returns a dict of the ini values.

        The datatype returned is a list of sections where each section is a
        dict of values.

        Args:

            filename: Full path to the ini-file to be parsed.

        Example the following the file::

            [main]
            name = this is a name
            key = value
            key = value2

        Would return::

            [
              {'main':
                {
                  'name': ['this is a name'],
                  'key': [value, value2]
                }
              },
            ]

        """
        if filename is None:
            return {}

        f = open(filename, encoding='utf-8')
        try:
            data = f.read()
            return self.parse_string(data)
        finally:
            f.close()

    def parse_string(self, string):
        """ Parses a string that is supposed to be ini-style format.

        See :py:meth:`parse_file` for more info

        Args:

            string: String to be parsed. Should be in ini-file format.

        Returns:

            Dictionnary containing all the sections of the ini-file and their
            respective data.

        Raises:

            :py:class:`ParserError` when line does not follow the ini format.

        """
        sections = {}
        # When parsing inside a section, the name of it stored here.
        section_name = None
        current_section = pynag.Utils.defaultdict(dict)

        for line_no, line, in enumerate(string.splitlines()):
            line = line.strip()

            # skip empty lines
            if not line or line[0] in ('#', ';'):
                continue

            # Check if this is a new section
            if line.startswith('[') and line.endswith(']'):
                section_name = line.strip('[').strip(']').strip()
                current_section = pynag.Utils.defaultdict(list)
                sections[section_name] = current_section
                continue

            # All entries should have key=value format
            if not '=' in line:
                error = "Line %s should be in the form of key=value format (got '%s' instead)" % (line_no, line)
                raise ParserError(error)

            # If we reach here, we parse current line into key and a value section
            key, value = line.split('=', 1)
            key = key.strip()
            value = value.strip()

            sections[section_name][key].append(value)
        return sections



# check_dns.py -- Returns OK if a hostname resolves to any ip address

# check_dns plugin will need some system libraries for DNS lookup
from _socket import gaierror
import socket
import time

# Import PluginHelper and some utility constants from the Plugins module
from pynag.Plugins import PluginHelper,ok,warning,critical,unknown

# Create an instance of PluginHelper()
my_plugin = PluginHelper()

# Our Plugin will need -H and -a attributes and we will use PluginHelpers wrapper around optparse for this:
my_plugin.parser.add_option('-H', help="Hostname or ip address", dest="hostname")
my_plugin.parser.add_option('-a', help="Expected Address", dest="address")

# When parse_arguments is called some default options like --threshold and --no-longoutput are automatically added
my_plugin.parse_arguments()


#
# Here starts Plugin-specific logic
#

# Get the hostname and expected address that was provided on the command-line
# address will be optional, but we will throw and error if hostname is not provided
hostname = my_plugin.options.hostname
address = my_plugin.options.address
if hostname is None:
    my_plugin.parser.error('-H argument is required')


# Here comes the specific check logic
try:
    start_time = time.time()
    result = socket.gethostbyname( hostname ) # result will contain the ip address resolved
    end_time = time.time()

    # If no address was specified with -a, then we return
    # OK if hostname resolved to anything at all
    if address is None or address == result:
        my_plugin.status(ok)
        my_plugin.add_summary("%s resolves to %s" % (hostname, result))
    else:
        my_plugin.status(critical)
        my_plugin.add_summary("%s resolves to %s but should resolve to %s" % (hostname,result,address))

    # Add run_time metric, so we can also alert if lookup takes to long
    run_time = end_time - start_time
    my_plugin.add_metric('run_time', run_time)
except gaierror:
    # If any exceptions happened in the code above, lets return a critical status
    my_plugin.status(critical)
    my_plugin.add_summary('Could not resolve host "%s"' % hostname )

# when check_all_metrics() is run, any metrics we have added with add_metric() will be processed against
# Thresholds (like --threshold). This part will allow our plugin users to alert on lookup_time
my_plugin.check_all_metrics()

# Print status output and exit
my_plugin.exit()



# check_load.py - Check load average. Thresholds can be specified from the commandline

# Import PluginHelper and some utility constants from the Plugins module
from pynag.Plugins import PluginHelper,ok,warning,critical,unknown

# Create an instance of PluginHelper()
helper = PluginHelper()

# Optionally, let helper handle command-line arguments for us for example --threshold
# Note: If your plugin needs any commandline arguments on its own (like --hostname) you should add them
# before this step with helper.parser.add_option()
helper.parse_arguments()


# Here starts our plugin specific logic. Lets try to read /proc/loadavg
# And if it fails, we exit immediately with UNKNOWN status
try:
    content = open('/proc/loadavg').read()
except Exception as e:
    helper.exit(summary="Could not read /proc/loadavg", long_output=str(e), exit_code=unknown, perfdata='')


# We have read the contents of loadavg file. Lets put it in the summary of our plugin output:
helper.add_summary("Load: %s" % content)


# Read metrics from /proc/loadavg and add them as performance metrics
load1,load5,load15,processes,last_proc_id = content.split()
running,total = processes.split('/')

# If we so desire we can set default thresholds by adding warn attribute here
# However we decide that there are no thresholds by default and they have to be
# applied on runtime with the --threshold option
helper.add_metric(label='load1',value=load1)
helper.add_metric(label='load5',value=load5)
helper.add_metric(label='load15',value=load15)
helper.add_metric(label='running_processes',value=running)
helper.add_metric(label='total_processes',value=total)

# By default assume everything is ok. Any thresholds specified with --threshold can overwrite this status:
helper.status(ok)

# Here all metrics will be checked against thresholds that are either
# built-in or added via --threshold from the command-line
helper.check_all_metrics()

# Print out plugin information and exit nagios-style
helper.exit()



#!/usr/bin/env python
import sys

if len(sys.argv) != 3:
    sys.stderr.write("Usage:  %s 'Host Name' 'Service Description'\n" % (sys.argv[0]))
    sys.exit(2)


## This is for the custom nagios module
sys.path.insert(1, '../')
from pynag.Parsers import config


## Create the plugin option
nc = config('/etc/nagios/nagios.cfg')
nc.parse()

service = nc.get_service(sys.argv[1],sys.argv[2])

print(nc.print_conf(service))



#!/usr/bin/env python

import sys

if len(sys.argv) != 3:
    sys.stderr.write("Usage:  %s 'Host Name' 'Hostgroup Name' \n" % (sys.argv[0]))
    sys.exit(2)

## This is for the custom nagios module
sys.path.insert(1, '../')
from pynag.Parsers import config

## Create the plugin option
nc = config('/etc/nagios/nagios.cfg')
nc.parse()


target_host = sys.argv[1]
target_group = sys.argv[2]


## Get the host object
host_obj = nc.get_host(target_host)
if not host_obj:
    sys.stderr.write("host_name '%s' does not exist\n" % target_host)
    sys.exit(2)

## Find the hostgroup from our global dictionaries
group_obj = nc.get_hostgroup(target_group)
if not group_obj:
    sys.stderr.write("%s does not exist\n" % target_group)
    sys.exit(2)

## Get a list of the host_name's in this group
existing_list = group_obj['members'].split(",")
if target_host not in existing_list:
    sys.stderr.write("%s is not in the group\n" % target_host)
    sys.exit(2)
else:
    existing_list.remove(target_host)

print("Removing %s from %s" % (target_host, target_group))

## Alphabetize the list, for easier readability (and to make it pretty)
existing_list.sort()

## Remove old group
nc['all_hostgroup'].remove(group_obj)

## Save the new member list
group_obj['members'] = ",".join(existing_list)

## Mark the commit flag for the group
group_obj['meta']['needs_commit'] = True

## Add the group back in with new members
nc['all_hostgroup'].append(group_obj)

## Commit the changes to file
nc.commit()



#!/usr/bin/env python
import sys

## This is for the custom nagios module
sys.path.insert(1, '../')
from pynag.Parsers import config

def is_ip(ip_address):
    import socket
    try:
        socket.inet_aton(ip_address)
        return True # We got through that call without an error, so it is valid
    except socket.error:
        return False # There was an error, so it is invalid

## Create the plugin option
nc = config('/etc/nagios/nagios.cfg')
nc.parse()


print("Checking hosts using dns names instead of ip addresses in the 'address' field")
for host in nc['all_host']:
    if 'address' in host:
        if not is_ip(host['address']):
            print("%s has a name instead of ip in the address field (%s)" % (host['alias'], host['address']))


print("Checking for weird service definitions")
for service in nc['all_service']:
    if 'register' in service and service['register'] == 0:
        continue
    if 'host_name' not in service:
        print(nc.print_conf( service ))





def add_host_comment(
    host_name,
    persistent,
    author,
    comment,
    command_file=None,
    timestamp=0
):
    """
    Adds a comment to a particular host.  If the "persistent" field
    is set to zero (0), the comment will be deleted the next time
    Nagios is restarted.  Otherwise, the comment will persist across
    program restarts until it is deleted manually.
    """
    return send_command("ADD_HOST_COMMENT",
                        command_file,
                        timestamp,
                        host_name,
                        persistent,
                        author,
                        comment)


def shutdown_program(
    command_file=None,
    timestamp=0
):
    """
    Shuts down the Nagios process.
    """
    return send_command("SHUTDOWN_PROGRAM",
                        command_file,
                        timestamp,
                        )


def disable_servicegroup_passive_svc_checks(
    servicegroup_name,
    command_file=None,
    timestamp=0
):
    """
    Disables the acceptance and processing of passive checks for all
    services in a particular servicegroup.
    """
    return send_command("DISABLE_SERVICEGROUP_PASSIVE_SVC_CHECKS",
                        command_file,
                        timestamp,
                        servicegroup_name)


def enable_servicegroup_passive_host_checks(
    servicegroup_name,
    command_file=None,
    timestamp=0
):
    """
    Enables the acceptance and processing of passive checks for all
    hosts that have services that are members of a particular
    service group.
    """
    return send_command("ENABLE_SERVICEGROUP_PASSIVE_HOST_CHECKS",
                        command_file,
                        timestamp,
                        servicegroup_name)


def disable_servicegroup_passive_host_checks(
    servicegroup_name,
    command_file=None,
    timestamp=0
):
    """
    Disables the acceptance and processing of passive checks for all
    hosts that have services that are members of a particular
    service group.
    """
    return send_command("DISABLE_SERVICEGROUP_PASSIVE_HOST_CHECKS",
                        command_file,
                        timestamp,
                        servicegroup_name)


def change_global_host_event_handler(
    event_handler_command,
    command_file=None,
    timestamp=0
):
    """
    Changes the global host event handler command to be that
    specified by the "event_handler_command" option.  The
    "event_handler_command" option specifies the short name of the
    command that should be used as the new host event handler.  The
    command must have been configured in Nagios before it was last
    (re)started.
    """
    return send_command("CHANGE_GLOBAL_HOST_EVENT_HANDLER",
                        command_file,
                        timestamp,
                        event_handler_command)


def change_global_svc_event_handler(
    event_handler_command,
    command_file=None,
    timestamp=0
):
    """
    Changes the global service event handler command to be that
    specified by the "event_handler_command" option.  The
    "event_handler_command" option specifies the short name of the
    command that should be used as the new service event handler.
    The command must have been configured in Nagios before it was
    last (re)started.
    """
    return send_command("CHANGE_GLOBAL_SVC_EVENT_HANDLER",
                        command_file,
                        timestamp,
                        event_handler_command)


def change_host_event_handler(
    host_name,
    event_handler_command,
    command_file=None,
    timestamp=0
):
    """
    Changes the event handler command for a particular host to be
    that specified by the "event_handler_command" option.  The
    "event_handler_command" option specifies the short name of the
    command that should be used as the new host event handler.  The
    command must have been configured in Nagios before it was last
    (re)started.
    """
    return send_command("CHANGE_HOST_EVENT_HANDLER",
                        command_file,
                        timestamp,
                        host_name,
                        event_handler_command)


def change_svc_event_handler(
    host_name,
    service_description,
    event_handler_command,
    command_file=None,
    timestamp=0
):
    """
    Changes the event handler command for a particular service to be
    that specified by the "event_handler_command" option.  The
    "event_handler_command" option specifies the short name of the
    command that should be used as the new service event handler.
    The command must have been configured in Nagios before it was
    last (re)started.
    """
    return send_command("CHANGE_SVC_EVENT_HANDLER",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        event_handler_command)


def change_host_check_command(
    host_name,
    check_command,
    command_file=None,
    timestamp=0
):
    """
    Changes the check command for a particular host to be that
    specified by the "check_command" option.  The "check_command"
    option specifies the short name of the command that should be
    used as the new host check command.  The command must have been
    configured in Nagios before it was last (re)started.
    """
    return send_command("CHANGE_HOST_CHECK_COMMAND",
                        command_file,
                        timestamp,
                        host_name,
                        check_command)


def change_svc_check_command(
    host_name,
    service_description,
    check_command,
    command_file=None,
    timestamp=0
):
    """
    Changes the check command for a particular service to be that
    specified by the "check_command" option.  The "check_command"
    option specifies the short name of the command that should be
    used as the new service check command.  The command must have
    been configured in Nagios before it was last (re)started.
    """
    return send_command("CHANGE_SVC_CHECK_COMMAND",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        check_command)


def change_normal_host_check_interval(
    host_name,
    check_interval,
    command_file=None,
    timestamp=0
):
    """
    Changes the normal (regularly scheduled) check interval for a
    particular host.
    """
    return send_command("CHANGE_NORMAL_HOST_CHECK_INTERVAL",
                        command_file,
                        timestamp,
                        host_name,
                        check_interval)


def enable_svc_notifications(
    host_name,
    service_description,
    command_file=None,
    timestamp=0
):
    """
    Enables notifications for a particular service.  Notifications
    will be sent out for the service only if notifications are
    enabled on a program-wide basis as well.
    """
    return send_command("ENABLE_SVC_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        host_name,
                        service_description)


def change_normal_svc_check_interval(
    host_name,
    service_description,
    check_interval,
    command_file=None,
    timestamp=0
):
    """
    Changes the normal (regularly scheduled) check interval for a
    particular service
    """
    return send_command("CHANGE_NORMAL_SVC_CHECK_INTERVAL",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        check_interval)


def change_retry_svc_check_interval(
    host_name,
    service_description,
    check_interval,
    command_file=None,
    timestamp=0
):
    """
    Changes the retry check interval for a particular service.
    """
    return send_command("CHANGE_RETRY_SVC_CHECK_INTERVAL",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        check_interval)


def change_max_host_check_attempts(
    host_name,
    check_attempts,
    command_file=None,
    timestamp=0
):
    """
    Changes the maximum number of check attempts (retries) for a
    particular host.
    """
    return send_command("CHANGE_MAX_HOST_CHECK_ATTEMPTS",
                        command_file,
                        timestamp,
                        host_name,
                        check_attempts)


def change_max_svc_check_attempts(
    host_name,
    service_description,
    check_attempts,
    command_file=None,
    timestamp=0
):
    """
    Changes the maximum number of check attempts (retries) for a
    particular service.
    """
    return send_command("CHANGE_MAX_SVC_CHECK_ATTEMPTS",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        check_attempts)


def process_service_check_result(
    host_name,
    service_description,
    return_code,
    plugin_output,
    command_file=None,
    timestamp=0
):
    """
    This is used to submit a passive check result for a particular
    service.  The "return_code" field should be one of the
    following: 0=OK, 1=WARNING, 2=CRITICAL, 3=UNKNOWN.  The
    "plugin_output" field contains text output from the service
    check, along with optional performance data.
    """
    return send_command("PROCESS_SERVICE_CHECK_RESULT",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        return_code,
                        plugin_output)


def process_host_check_result(
    host_name,
    status_code,
    plugin_output,
    command_file=None,
    timestamp=0
):
    """
    This is used to submit a passive check result for a particular
    host.  The "status_code" indicates the state of the host check
    and should be one of the following: 0=UP, 1=DOWN, 2=UNREACHABLE.
    The "plugin_output" argument contains the text returned from the
    host check, along with optional performance data.
    """
    return send_command("PROCESS_HOST_CHECK_RESULT",
                        command_file,
                        timestamp,
                        host_name,
                        status_code,
                        plugin_output)


def remove_host_acknowledgement(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    This removes the problem acknowledgement for a particular host.
    Once the acknowledgement has been removed, notifications can
    once again be sent out for the given host.
    """
    return send_command("REMOVE_HOST_ACKNOWLEDGEMENT",
                        command_file,
                        timestamp,
                        host_name)


def remove_svc_acknowledgement(
    host_name,
    service_description,
    command_file=None,
    timestamp=0
):
    """
    This removes the problem acknowledgement for a particular
    service.  Once the acknowledgement has been removed,
    notifications can once again be sent out for the given service.
    """
    return send_command("REMOVE_SVC_ACKNOWLEDGEMENT",
                        command_file,
                        timestamp,
                        host_name,
                        service_description)


def schedule_host_downtime(
    host_name,
    start_time,
    end_time,
    fixed,
    trigger_id,
    duration,
    author,
    comment,
    command_file=None,
    timestamp=0
):
    """
    Schedules downtime for a specified host.  If the "fixed"
    argument is set to one (1), downtime will start and end at the
    times specified by the "start" and "end" arguments.  Otherwise,
    downtime will begin between the "start" and "end" times and last
    for "duration" seconds.  The "start" and "end" arguments are
    specified in time_t format (seconds since the UNIX epoch).  The
    specified host downtime can be triggered by another downtime
    entry if the "trigger_id" is set to the ID of another scheduled
    downtime entry.  Set the "trigger_id" argument to zero (0) if
    the downtime for the specified host should not be triggered by
    another downtime entry.
    """
    return send_command("SCHEDULE_HOST_DOWNTIME",
                        command_file,
                        timestamp,
                        host_name,
                        start_time,
                        end_time,
                        fixed,
                        trigger_id,
                        duration,
                        author,
                        comment)


def schedule_svc_downtime(
    host_name,
    service_description,
    start_time,
    end_time,
    fixed,
    trigger_id,
    duration,
    author,
    comment,
    command_file=None,
    timestamp=0
):
    """
    Schedules downtime for a specified service.  If the "fixed"
    argument is set to one (1), downtime will start and end at the
    times specified by the "start" and "end" arguments.  Otherwise,
    downtime will begin between the "start" and "end" times and last
    for "duration" seconds.  The "start" and "end" arguments are
    specified in time_t format (seconds since the UNIX epoch).  The
    specified service downtime can be triggered by another downtime
    entry if the "trigger_id" is set to the ID of another scheduled
    downtime entry.  Set the "trigger_id" argument to zero (0) if
    the downtime for the specified service should not be triggered
    by another downtime entry.
    """
    return send_command("SCHEDULE_SVC_DOWNTIME",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        start_time,
                        end_time,
                        fixed,
                        trigger_id,
                        duration,
                        author,
                        comment)


def disable_svc_notifications(
    host_name,
    service_description,
    command_file=None,
    timestamp=0
):
    """
    Disables notifications for a particular service.
    """
    return send_command("DISABLE_SVC_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        host_name,
                        service_description)


def schedule_servicegroup_svc_downtime(
    servicegroup_name,
    start_time,
    end_time,
    fixed,
    trigger_id,
    duration,
    author,
    comment,
    command_file=None,
    timestamp=0
):
    """
    Schedules downtime for all services in a specified servicegroup.
    If the "fixed" argument is set to one (1), downtime will start
    and end at the times specified by the "start" and "end"
    arguments.  Otherwise, downtime will begin between the "start"
    and "end" times and last for "duration" seconds.  The "start"
    and "end" arguments are specified in time_t format (seconds
    since the UNIX epoch).  The service downtime entries can be
    triggered by another downtime entry if the "trigger_id" is set
    to the ID of another scheduled downtime entry.  Set the
    "trigger_id" argument to zero (0) if the downtime for the
    services should not be triggered by another downtime entry.
    """
    return send_command("SCHEDULE_SERVICEGROUP_SVC_DOWNTIME",
                        command_file,
                        timestamp,
                        servicegroup_name,
                        start_time,
                        end_time,
                        fixed,
                        trigger_id,
                        duration,
                        author,
                        comment)


def schedule_servicegroup_host_downtime(
    servicegroup_name,
    start_time,
    end_time,
    fixed,
    trigger_id,
    duration,
    author,
    comment,
    command_file=None,
    timestamp=0
):
    """
    Schedules downtime for all hosts that have services in a
    specified servicegroup.  If the "fixed" argument is set to one
    (1), downtime will start and end at the times specified by the
    "start" and "end" arguments.  Otherwise, downtime will begin
    between the "start" and "end" times and last for "duration"
    seconds.  The "start" and "end" arguments are specified in
    time_t format (seconds since the UNIX epoch).  The host downtime
    entries can be triggered by another downtime entry if the
    "trigger_id" is set to the ID of another scheduled downtime
    entry.  Set the "trigger_id" argument to zero (0) if the
    downtime for the hosts should not be triggered by another
    downtime entry.
    """
    return send_command("SCHEDULE_SERVICEGROUP_HOST_DOWNTIME",
                        command_file,
                        timestamp,
                        servicegroup_name,
                        start_time,
                        end_time,
                        fixed,
                        trigger_id,
                        duration,
                        author,
                        comment)


def schedule_host_svc_downtime(
    host_name,
    start_time,
    end_time,
    fixed,
    trigger_id,
    duration,
    author,
    comment,
    command_file=None,
    timestamp=0
):
    """
    Schedules downtime for all services associated with a particular
    host.  If the "fixed" argument is set to one (1), downtime will
    start and end at the times specified by the "start" and "end"
    arguments.  Otherwise, downtime will begin between the "start"
    and "end" times and last for "duration" seconds.  The "start"
    and "end" arguments are specified in time_t format (seconds
    since the UNIX epoch).  The service downtime entries can be
    triggered by another downtime entry if the "trigger_id" is set
    to the ID of another scheduled downtime entry.  Set the
    "trigger_id" argument to zero (0) if the downtime for the
    services should not be triggered by another downtime entry.
    """
    return send_command("SCHEDULE_HOST_SVC_DOWNTIME",
                        command_file,
                        timestamp,
                        host_name,
                        start_time,
                        end_time,
                        fixed,
                        trigger_id,
                        duration,
                        author,
                        comment)


def schedule_hostgroup_host_downtime(
    hostgroup_name,
    start_time,
    end_time,
    fixed,
    trigger_id,
    duration,
    author,
    comment,
    command_file=None,
    timestamp=0
):
    """
    Schedules downtime for all hosts in a specified hostgroup.  If
    the "fixed" argument is set to one (1), downtime will start and
    end at the times specified by the "start" and "end" arguments.
    Otherwise, downtime will begin between the "start" and "end"
    times and last for "duration" seconds.  The "start" and "end"
    arguments are specified in time_t format (seconds since the UNIX
    epoch).  The host downtime entries can be triggered by another
    downtime entry if the "trigger_id" is set to the ID of another
    scheduled downtime entry.  Set the "trigger_id" argument to zero
    (0) if the downtime for the hosts should not be triggered by
    another downtime entry.
    """
    return send_command("SCHEDULE_HOSTGROUP_HOST_DOWNTIME",
                        command_file,
                        timestamp,
                        hostgroup_name,
                        start_time,
                        end_time,
                        fixed,
                        trigger_id,
                        duration,
                        author,
                        comment)


def schedule_hostgroup_svc_downtime(
    hostgroup_name,
    start_time,
    end_time,
    fixed,
    trigger_id,
    duration,
    author,
    comment,
    command_file=None,
    timestamp=0
):
    """
    Schedules downtime for all services associated with hosts in a
    specified servicegroup.  If the "fixed" argument is set to one
    (1), downtime will start and end at the times specified by the
    "start" and "end" arguments.  Otherwise, downtime will begin
    between the "start" and "end" times and last for "duration"
    seconds.  The "start" and "end" arguments are specified in
    time_t format (seconds since the UNIX epoch).  The service
    downtime entries can be triggered by another downtime entry if
    the "trigger_id" is set to the ID of another scheduled downtime
    entry.  Set the "trigger_id" argument to zero (0) if the
    downtime for the services should not be triggered by another
    downtime entry.
    """
    return send_command("SCHEDULE_HOSTGROUP_SVC_DOWNTIME",
                        command_file,
                        timestamp,
                        hostgroup_name,
                        start_time,
                        end_time,
                        fixed,
                        trigger_id,
                        duration,
                        author,
                        comment)


def del_host_downtime(
    downtime_id,
    command_file=None,
    timestamp=0
):
    """
    Deletes the host downtime entry that has an ID number matching
    the "downtime_id" argument.  If the downtime is currently in
    effect, the host will come out of scheduled downtime (as long as
    there are no other overlapping active downtime entries).
    """
    return send_command("DEL_HOST_DOWNTIME",
                        command_file,
                        timestamp,
                        downtime_id)


def del_svc_downtime(
    downtime_id,
    command_file=None,
    timestamp=0
):
    """
    Deletes the service downtime entry that has an ID number
    matching the "downtime_id" argument.  If the downtime is
    currently in effect, the service will come out of scheduled
    downtime (as long as there are no other overlapping active
    downtime entries).
    """
    return send_command("DEL_SVC_DOWNTIME",
                        command_file,
                        timestamp,
                        downtime_id)


def schedule_host_check(
    host_name,
    check_time,
    command_file=None,
    timestamp=0
):
    """
    Schedules the next active check of a particular host at
    "check_time".  The "check_time" argument is specified in time_t
    format (seconds since the UNIX epoch).  Note that the host may
    not actually be checked at the time you specify.  This could
    occur for a number of reasons: active checks are disabled on a
    program-wide or service-specific basis, the host is already
    scheduled to be checked at an earlier time, etc.  If you want to
    force the host check to occur at the time you specify, look at
    the SCHEDULE_FORCED_HOST_CHECK command.
    """
    return send_command("SCHEDULE_HOST_CHECK",
                        command_file,
                        timestamp,
                        host_name,
                        check_time)


def schedule_forced_host_check(
    host_name,
    check_time,
    command_file=None,
    timestamp=0
):
    """
    Schedules a forced active check of a particular host at
    "check_time".  The "check_time" argument is specified in time_t
    format (seconds since the UNIX epoch).   Forced checks are
    performed regardless of what time it is (e.g. timeperiod
    restrictions are ignored) and whether or not active checks are
    enabled on a host-specific or program-wide basis.
    """
    return send_command("SCHEDULE_FORCED_HOST_CHECK",
                        command_file,
                        timestamp,
                        host_name,
                        check_time)


def schedule_forced_svc_check(
    host_name,
    service_description,
    check_time,
    command_file=None,
    timestamp=0
):
    """
    Schedules a forced active check of a particular service at
    "check_time".  The "check_time" argument is specified in time_t
    format (seconds since the UNIX epoch).   Forced checks are
    performed regardless of what time it is (e.g. timeperiod
    restrictions are ignored) and whether or not active checks are
    enabled on a service-specific or program-wide basis.
    """
    return send_command("SCHEDULE_FORCED_SVC_CHECK",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        check_time)


def del_all_host_comments(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Deletes all comments assocated with a particular host.
    """
    return send_command("DEL_ALL_HOST_COMMENTS",
                        command_file,
                        timestamp,
                        host_name)


def schedule_forced_host_svc_checks(
    host_name,
    check_time,
    command_file=None,
    timestamp=0
):
    """
    Schedules a forced active check of all services associated with
    a particular host at "check_time".  The "check_time" argument is
    specified in time_t format (seconds since the UNIX epoch).
    Forced checks are performed regardless of what time it is (e.g.
    timeperiod restrictions are ignored) and whether or not active
    checks are enabled on a service-specific or program-wide basis.
    """
    return send_command("SCHEDULE_FORCED_HOST_SVC_CHECKS",
                        command_file,
                        timestamp,
                        host_name,
                        check_time)


def process_file(
    file_name,
    delete,
    command_file=None,
    timestamp=0
):
    """
    Directs Nagios to process all external commands that are found
    in the file specified by the <file_name> argument.  If the
    <delete> option is non-zero, the file will be deleted once it
    has been processes.  If the <delete> option is set to zero, the
    file is left untouched.
    """
    return send_command("PROCESS_FILE",
                        command_file,
                        timestamp,
                        file_name,
                        delete)


def change_host_check_timeperiod(
    host_name,
    check_timeperod,
    command_file=None,
    timestamp=0
):
    """
    Changes the check timeperiod for a particular host to what is
    specified by the "check_timeperiod" option.  The
    "check_timeperiod" option should be the short name of the
    timeperod that is to be used as the host check timeperiod.  The
    timeperiod must have been configured in Nagios before it was
    last (re)started.
    """
    return send_command("CHANGE_HOST_CHECK_TIMEPERIOD",
                        command_file,
                        timestamp,
                        host_name,
                        check_timeperod)


def send_custom_host_notification(
    host_name,
    options,
    author,
    comment,
    command_file=None,
    timestamp=0
):
    """
    Allows you to send a custom host notification.  Very useful in
    dire situations, emergencies or to communicate with all admins
    that are responsible for a particular host.  When the host
    notification is sent out, the $NOTIFICATIONTYPE$ macro will be
    set to "CUSTOM".  The <options> field is a logical OR of the
    following integer values that affect aspects of the notification
    that are sent out: 0 = No option (default), 1 = Broadcast (send
    notification to all normal and all escalated contacts for the
    host), 2 = Forced (notification is sent out regardless of
    current time, whether or not notifications are enabled, etc.), 4
    = Increment current notification # for the host (this is not
    done by default for custom notifications).  The comment field
    can be used with the
    $NOTIFICATIONCOMMENT$ macro in notification commands.
    """
    return send_command("SEND_CUSTOM_HOST_NOTIFICATION",
                        command_file,
                        timestamp,
                        host_name,
                        options,
                        author,
                        comment)


def send_custom_svc_notification(
    host_name,
    service_description,
    options,
    author,
    comment,
    command_file=None,
    timestamp=0
):
    """
    Allows you to send a custom service notification.  Very useful
    in dire situations, emergencies or to communicate with all
    admins that are responsible for a particular service.  When the
    service notification is sent out, the $NOTIFICATIONTYPE$ macro
    will be set to "CUSTOM".  The <options> field is a logical OR of
    the following integer values that affect aspects of the
    notification that are sent out: 0 = No option (default), 1 =
    Broadcast (send notification to all normal and all escalated
    contacts for the service), 2 = Forced (notification is sent out
    regardless of current time, whether or not notifications are
    enabled, etc.), 4 = Increment current notification # for the
    service(this is not done by default for custom notifications).
    The comment field can be used with the
    $NOTIFICATIONCOMMENT$ macro in notification commands.
    """
    return send_command("SEND_CUSTOM_SVC_NOTIFICATION",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        options,
                        author,
                        comment)


def change_retry_host_check_interval(
    host_name,
    service_description,
    check_interval,
    command_file=None,
    timestamp=0
):
    """
    Changes the retry check interval for a particular host.
    """
    return send_command("CHANGE_RETRY_HOST_CHECK_INTERVAL",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        check_interval)


def change_svc_check_timeperiod(
    host_name,
    service_description,
    check_timeperiod,
    command_file=None,
    timestamp=0
):
    """
    Changes the check timeperiod for a particular service to what is
    specified by the "check_timeperiod" option.  The
    "check_timeperiod" option should be the short name of the
    timeperod that is to be used as the service check timeperiod.
    The timeperiod must have been configured in Nagios before it was
    last (re)started.
    """
    return send_command("CHANGE_SVC_CHECK_TIMEPERIOD",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        check_timeperiod)


def change_host_check_timeperiod(
    host_name,
    timeperiod,
    command_file=None,
    timestamp=0
):
    """
    Changes the valid check period for the specified host.
    """
    return send_command("CHANGE_HOST_CHECK_TIMEPERIOD",
                        command_file,
                        timestamp,
                        host_name,
                        timeperiod)


def change_custom_host_var(
    host_name,
    varname,
    varvalue,
    command_file=None,
    timestamp=0
):
    """
    Changes the value of a custom host variable.
    """
    return send_command("CHANGE_CUSTOM_HOST_VAR",
                        command_file,
                        timestamp,
                        host_name,
                        varname,
                        varvalue)


def del_all_svc_comments(
    host_name,
    service_description,
    command_file=None,
    timestamp=0
):
    """
    Deletes all comments associated with a particular service.
    """
    return send_command("DEL_ALL_SVC_COMMENTS",
                        command_file,
                        timestamp,
                        host_name,
                        service_description)


def change_custom_svc_var(
    host_name,
    service_description,
    varname,
    varvalue,
    command_file=None,
    timestamp=0
):
    """
    Changes the value of a custom service variable.
    """
    return send_command("CHANGE_CUSTOM_SVC_VAR",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        varname,
                        varvalue)


def change_custom_contact_var(
    contact_name,
    varname,
    varvalue,
    command_file=None,
    timestamp=0
):
    """
    Changes the value of a custom contact variable.
    """
    return send_command("CHANGE_CUSTOM_CONTACT_VAR",
                        command_file,
                        timestamp,
                        contact_name,
                        varname,
                        varvalue)


def enable_contact_host_notifications(
    contact_name,
    command_file=None,
    timestamp=0
):
    """
    Enables host notifications for a particular contact.
    """
    return send_command("ENABLE_CONTACT_HOST_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        contact_name)


def disable_contact_host_notifications(
    contact_name,
    command_file=None,
    timestamp=0
):
    """
    Disables host notifications for a particular contact.
    """
    return send_command("DISABLE_CONTACT_HOST_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        contact_name)


def enable_contact_svc_notifications(
    contact_name,
    command_file=None,
    timestamp=0
):
    """
    Disables service notifications for a particular contact.
    """
    return send_command("ENABLE_CONTACT_SVC_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        contact_name)


def disable_contact_svc_notifications(
    contact_name,
    command_file=None,
    timestamp=0
):
    """
    Disables service notifications for a particular contact.
    """
    return send_command("DISABLE_CONTACT_SVC_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        contact_name)


def enable_contactgroup_host_notifications(
    contactgroup_name,
    command_file=None,
    timestamp=0
):
    """
    Enables host notifications for all contacts in a particular
    contactgroup.
    """
    return send_command("ENABLE_CONTACTGROUP_HOST_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        contactgroup_name)


def disable_contactgroup_host_notifications(
    contactgroup_name,
    command_file=None,
    timestamp=0
):
    """
    Disables host notifications for all contacts in a particular
    contactgroup.
    """
    return send_command("DISABLE_CONTACTGROUP_HOST_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        contactgroup_name)


def enable_contactgroup_svc_notifications(
    contactgroup_name,
    command_file=None,
    timestamp=0
):
    """
    Enables service notifications for all contacts in a particular
    contactgroup.
    """
    return send_command("ENABLE_CONTACTGROUP_SVC_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        contactgroup_name)


def disable_contactgroup_svc_notifications(
    contactgroup_name,
    command_file=None,
    timestamp=0
):
    """
    Disables service notifications for all contacts in a particular
    contactgroup.
    """
    return send_command("DISABLE_CONTACTGROUP_SVC_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        contactgroup_name)


def enable_host_notifications(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Enables notifications for a particular host.  Notifications will
    be sent out for the host only if notifications are enabled on a
    program-wide basis as well.
    """
    return send_command("ENABLE_HOST_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        host_name)


def disable_svc_flap_detection(
    host_name,
    service_description,
    command_file=None,
    timestamp=0
):
    """
    Disables flap detection for the specified service.
    """
    return send_command("DISABLE_SVC_FLAP_DETECTION",
                        command_file,
                        timestamp,
                        host_name,
                        service_description)


def change_svc_notification_timeperiod(
    host_name,
    service_description,
    notification_timeperiod,
    command_file=None,
    timestamp=0
):
    """
    Changes the notification timeperiod for a particular service to
    what is specified by the "notification_timeperiod" option.  The
    "notification_timeperiod" option should be the short name of the
    timeperiod that is to be used as the service notification
    timeperiod.  The timeperiod must have been configured in Nagios
    before it was last (re)started.
    """
    return send_command("CHANGE_SVC_NOTIFICATION_TIMEPERIOD",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        notification_timeperiod)


def change_contact_svc_notification_timeperiod(
    contact_name,
    notification_timeperiod,
    command_file=None,
    timestamp=0
):
    """
    Changes the service notification timeperiod for a particular
    contact to what is specified by the "notification_timeperiod"
    option.  The "notification_timeperiod" option should be the
    short name of the timeperiod that is to be used as the contact's
    service notification timeperiod.  The timeperiod must have been
    configured in Nagios before it was last (re)started.
    """
    return send_command("CHANGE_CONTACT_SVC_NOTIFICATION_TIMEPERIOD",
                        command_file,
                        timestamp,
                        contact_name,
                        notification_timeperiod)


def change_contact_host_notification_timeperiod(
    contact_name,
    notification_timeperiod,
    command_file=None,
    timestamp=0
):
    """
    Changes the host notification timeperiod for a particular
    contact to what is specified by the "notification_timeperiod"
    option.  The "notification_timeperiod" option should be the
    short name of the timeperiod that is to be used as the contact's
    host notification timeperiod.  The timeperiod must have been
    configured in Nagios before it was last (re)started.
    """
    return send_command("CHANGE_CONTACT_HOST_NOTIFICATION_TIMEPERIOD",
                        command_file,
                        timestamp,
                        contact_name,
                        notification_timeperiod)


def change_host_modattr(
    host_name,
    value,
    command_file=None,
    timestamp=0
):
    """
    This command changes the modified attributes value for the
    specified host.  Modified attributes values are used by Nagios
    to determine which object properties should be retained across
    program restarts.  Thus, modifying the value of the attributes
    can affect data retention.  This is an advanced option and
    should only be used by people who are intimately familiar with
    the data retention logic in Nagios.
    """
    return send_command("CHANGE_HOST_MODATTR",
                        command_file,
                        timestamp,
                        host_name,
                        value)


def change_svc_modattr(
    host_name,
    service_description,
    value,
    command_file=None,
    timestamp=0
):
    """
    This command changes the modified attributes value for the
    specified service.  Modified attributes values are used by
    Nagios to determine which object properties should be retained
    across program restarts.  Thus, modifying the value of the
    attributes can affect data retention.  This is an advanced
    option and should only be used by people who are intimately
    familiar with the data retention logic in Nagios.
    """
    return send_command("CHANGE_SVC_MODATTR",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        value)


def change_contact_modattr(
    contact_name,
    value,
    command_file=None,
    timestamp=0
):
    """
    This command changes the modified attributes value for the
    specified contact.  Modified attributes values are used by
    Nagios to determine which object properties should be retained
    across program restarts.  Thus, modifying the value of the
    attributes can affect data retention.  This is an advanced
    option and should only be used by people who are intimately
    familiar with the data retention logic in Nagios.
    """
    return send_command("CHANGE_CONTACT_MODATTR",
                        command_file,
                        timestamp,
                        contact_name,
                        value)


def change_contact_modhattr(
    contact_name,
    value,
    command_file=None,
    timestamp=0
):
    """
    This command changes the modified host attributes value for the
    specified contact.  Modified attributes values are used by
    Nagios to determine which object properties should be retained
    across program restarts.  Thus, modifying the value of the
    attributes can affect data retention.  This is an advanced
    option and should only be used by people who are intimately
    familiar with the data retention logic in Nagios.
    """
    return send_command("CHANGE_CONTACT_MODHATTR",
                        command_file,
                        timestamp,
                        contact_name,
                        value)


def change_contact_modsattr(
    contact_name,
    value,
    command_file=None,
    timestamp=0
):
    """
    This command changes the modified service attributes value for
    the specified contact.  Modified attributes values are used by
    Nagios to determine which object properties should be retained
    across program restarts.  Thus, modifying the value of the
    attributes can affect data retention.  This is an advanced
    option and should only be used by people who are intimately
    familiar with the data retention logic in Nagios.
    """
    return send_command("CHANGE_CONTACT_MODSATTR",
                        command_file,
                        timestamp,
                        contact_name,
                        value)


def disable_host_notifications(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Disables notifications for a particular host.
    """
    return send_command("DISABLE_HOST_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        host_name)


def enable_all_notifications_beyond_host(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Enables notifications for all hosts and services "beyond" (e.g.
    on all child hosts of) the specified host.  The current
    notification setting for the specified host is not affected.
    Notifications will only be sent out for these hosts and services
    if notifications are also enabled on a program-wide basis.
    """
    return send_command("ENABLE_ALL_NOTIFICATIONS_BEYOND_HOST",
                        command_file,
                        timestamp,
                        host_name)


def disable_all_notifications_beyond_host(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Disables notifications for all hosts and services "beyond" (e.g.
    on all child hosts of) the specified host.  The current
    notification setting for the specified host is not affected.
    """
    return send_command("DISABLE_ALL_NOTIFICATIONS_BEYOND_HOST",
                        command_file,
                        timestamp,
                        host_name)


def enable_host_and_child_notifications(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Enables notifications for the specified host, as well as all
    hosts "beyond" (e.g. on all child hosts of) the specified host.
    Notifications will only be sent out for these hosts if
    notifications are also enabled on a program-wide basis.
    """
    return send_command("ENABLE_HOST_AND_CHILD_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        host_name)


def add_svc_comment(
    host_name,
    service_description,
    persistent,
    author,
    comment,
    command_file=None,
    timestamp=0
):
    """
    Adds a comment to a particular service.  If the "persistent"
    field is set to zero (0), the comment will be deleted the next
    time Nagios is restarted.  Otherwise, the comment will persist
    across program restarts until it is deleted manually.
    """
    return send_command("ADD_SVC_COMMENT",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        persistent,
                        author,
                        comment)


def disable_host_and_child_notifications(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Disables notifications for the specified host, as well as all
    hosts "beyond" (e.g. on all child hosts of) the specified host.
    """
    return send_command("DISABLE_HOST_AND_CHILD_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        host_name)


def set_host_notification_number(
    host_name,
    notification_number,
    command_file=None,
    timestamp=0
):
    """
    Sets the current notification number for a particular host.  A
    value of 0 indicates that no notification has yet been sent for
    the current host problem.  Useful for forcing an escalation
    (based on notification number) or replicating notification
    information in redundant monitoring environments. Notification
    numbers greater than zero have no noticeable affect on the
    notification process if the host is currently in an UP state.
    """
    return send_command("SET_HOST_NOTIFICATION_NUMBER",
                        command_file,
                        timestamp,
                        host_name,
                        notification_number)


def set_svc_notification_number(
    host_name,
    service_description,
    notification_number,
    command_file=None,
    timestamp=0
):
    """
    Sets the current notification number for a particular service.
    A value of 0 indicates that no notification has yet been sent
    for the current service problem.  Useful for forcing an
    escalation (based on notification number) or replicating
    notification information in redundant monitoring environments.
    Notification numbers greater than zero have no noticeable affect
    on the notification process if the service is currently in an OK
    state.
    """
    return send_command("SET_SVC_NOTIFICATION_NUMBER",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        notification_number)


def enable_service_freshness_checks(
    command_file=None,
    timestamp=0
):
    """
    Enables freshness checks of all services on a program-wide
    basis.  Individual services that have freshness checks disabled
    will not be checked for freshness.
    """
    return send_command("ENABLE_SERVICE_FRESHNESS_CHECKS",
                        command_file,
                        timestamp,
                        )


def enable_host_freshness_checks(
    command_file=None,
    timestamp=0
):
    """
    Enables freshness checks of all hosts on a program-wide basis.
    Individual hosts that have freshness checks disabled will not be
    checked for freshness.
    """
    return send_command("ENABLE_HOST_FRESHNESS_CHECKS",
                        command_file,
                        timestamp,
                        )


def disable_service_freshness_checks(
    command_file=None,
    timestamp=0
):
    """
    Disables freshness checks of all services on a program-wide
    basis.
    """
    return send_command("DISABLE_SERVICE_FRESHNESS_CHECKS",
                        command_file,
                        timestamp,
                        )


def disable_host_freshness_checks(
    command_file=None,
    timestamp=0
):
    """
    Disables freshness checks of all hosts on a program-wide basis.
    """
    return send_command("DISABLE_HOST_FRESHNESS_CHECKS",
                        command_file,
                        timestamp,
                        )


def schedule_and_propagate_triggered_host_downtime(
    host_name,
    start_time,
    end_time,
    fixed,
    trigger_id,
    duration,
    author,
    comment,
    command_file=None,
    timestamp=0
):
    """
    Schedules downtime for a specified host and all of its children
    (hosts).  If the "fixed" argument is set to one (1), downtime
    will start and end at the times specified by the "start" and
    "end" arguments.  Otherwise, downtime will begin between the
    "start" and "end" times and last for "duration" seconds.  The
    "start" and "end" arguments are specified in time_t format
    (seconds since the UNIX epoch).  Downtime for child hosts are
    all set to be triggered by the downtime for the specified
    (parent) host.  The specified (parent) host downtime can be
    triggered by another downtime entry if the "trigger_id" is set
    to the ID of another scheduled downtime entry.  Set the
    "trigger_id" argument to zero (0) if the downtime for the
    specified (parent) host should not be triggered by another
    downtime entry.
    """
    return send_command("SCHEDULE_AND_PROPAGATE_TRIGGERED_HOST_DOWNTIME",
                        command_file,
                        timestamp,
                        host_name,
                        start_time,
                        end_time,
                        fixed,
                        trigger_id,
                        duration,
                        author,
                        comment)


def schedule_and_propagate_host_downtime(
    host_name,
    start_time,
    end_time,
    fixed,
    trigger_id,
    duration,
    author,
    comment,
    command_file=None,
    timestamp=0
):
    """
    Schedules downtime for a specified host and all of its children
    (hosts).  If the "fixed" argument is set to one (1), downtime
    will start and end at the times specified by the "start" and
    "end" arguments.  Otherwise, downtime will begin between the
    "start" and "end" times and last for "duration" seconds.  The
    "start" and "end" arguments are specified in time_t format
    (seconds since the UNIX epoch).  The specified (parent) host
    downtime can be triggered by another downtime entry if the
    "trigger_id" is set to the ID of another scheduled downtime
    entry.  Set the "trigger_id" argument to zero (0) if the
    downtime for the specified (parent) host should not be triggered
    by another downtime entry.
    """
    return send_command("SCHEDULE_AND_PROPAGATE_HOST_DOWNTIME",
                        command_file,
                        timestamp,
                        host_name,
                        start_time,
                        end_time,
                        fixed,
                        trigger_id,
                        duration,
                        author,
                        comment)


def schedule_svc_check(
    host_name,
    service_description,
    check_time,
    command_file=None,
    timestamp=0
):
    """
    Schedules the next active check of a specified service at
    "check_time".  The "check_time" argument is specified in time_t
    format (seconds since the UNIX epoch).  Note that the service
    may not actually be checked at the time you specify.  This could
    occur for a number of reasons: active checks are disabled on a
    program-wide or service-specific basis, the service is already
    scheduled to be checked at an earlier time, etc.  If you want to
    force the service check to occur at the time you specify, look
    at the SCHEDULE_FORCED_SVC_CHECK command.
    """
    return send_command("SCHEDULE_SVC_CHECK",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        check_time)


def del_host_comment(
    comment_id,
    command_file=None,
    timestamp=0
):
    """
    Deletes a host comment.  The id number of the comment that is to
    be deleted must be specified.
    """
    return send_command("DEL_HOST_COMMENT",
                        command_file,
                        timestamp,
                        comment_id)


def schedule_host_svc_checks(
    host_name,
    check_time,
    command_file=None,
    timestamp=0
):
    """
    Schedules the next active check of all services on a particular
    host at "check_time".  The "check_time" argument is specified in
    time_t format (seconds since the UNIX epoch).  Note that the
    services may not actually be checked at the time you specify.
    This could occur for a number of reasons: active checks are
    disabled on a program-wide or service-specific basis, the
    services are already scheduled to be checked at an earlier time,
    etc.  If you want to force the service checks to occur at the
    time you specify, look at the SCHEDULE_FORCED_HOST_SVC_CHECKS
    command.
    """
    return send_command("SCHEDULE_HOST_SVC_CHECKS",
                        command_file,
                        timestamp,
                        host_name,
                        check_time)


def save_state_information(
    command_file=None,
    timestamp=0
):
    """
    Causes Nagios to save all current monitoring status information
    to the state retention file.  Normally, state retention
    information is saved before the Nagios process shuts down and
    (potentially) at regularly scheduled intervals.  This command
    allows you to force Nagios to save this information to the state
    retention file immediately.  This does not affect the current
    status information in the Nagios process.
    """
    return send_command("SAVE_STATE_INFORMATION",
                        command_file,
                        timestamp,
                        )


def read_state_information(
    command_file=None,
    timestamp=0
):
    """
    Causes Nagios to load all current monitoring status information
    from the state retention file.  Normally, state retention
    information is loaded when the Nagios process starts up and
    before it starts monitoring.  WARNING: This command will cause
    Nagios to discard all current monitoring status information and
    use the information stored in state retention file!  Use with
    care.
    """
    return send_command("READ_STATE_INFORMATION",
                        command_file,
                        timestamp,
                        )


def enable_host_svc_checks(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Enables active checks of all services on the specified host.
    """
    return send_command("ENABLE_HOST_SVC_CHECKS",
                        command_file,
                        timestamp,
                        host_name)


def disable_host_svc_checks(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Enables active checks of all services on the specified host.
    """
    return send_command("DISABLE_HOST_SVC_CHECKS",
                        command_file,
                        timestamp,
                        host_name)


def enable_host_svc_notifications(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Enables notifications for all services on the specified host.
    Note that notifications will not be sent out if notifications
    are disabled on a program-wide basis.
    """
    return send_command("ENABLE_HOST_SVC_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        host_name)


def disable_host_svc_notifications(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Disables notifications for all services on the specified host.
    """
    return send_command("DISABLE_HOST_SVC_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        host_name)


def delay_svc_notification(
    host_name,
    service_description,
    notification_time,
    command_file=None,
    timestamp=0
):
    """
    Delays the next notification for a parciular service until
    "notification_time".  The "notification_time" argument is
    specified in time_t format (seconds since the UNIX epoch).  Note
    that this will only have an affect if the service stays in the
    same problem state that it is currently in.  If the service
    changes to another state, a new notification may go out before
    the time you specify in the "notification_time" argument.
    """
    return send_command("DELAY_SVC_NOTIFICATION",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        notification_time)


def delay_host_notification(
    host_name,
    notification_time,
    command_file=None,
    timestamp=0
):
    """
    Delays the next notification for a parciular service until
    "notification_time".  The "notification_time" argument is
    specified in time_t format (seconds since the UNIX epoch).  Note
    that this will only have an affect if the service stays in the
    same problem state that it is currently in.  If the service
    changes to another state, a new notification may go out before
    the time you specify in the "notification_time" argument.
    """
    return send_command("DELAY_HOST_NOTIFICATION",
                        command_file,
                        timestamp,
                        host_name,
                        notification_time)


def acknowledge_host_problem(
    host_name,
    sticky,
    notify,
    persistent,
    author,
    comment,
    command_file=None,
    timestamp=0
):
    """
    Allows you to acknowledge the current problem for the specified
    host.  By acknowledging the current problem, future
    notifications (for the same host state) are disabled.  If the
    "sticky" option is set to two (2), the acknowledgement will
    remain until the host returns to an UP state.  Otherwise the
    acknowledgement will automatically be removed when the host
    changes state.  If the "notify" option is set to one (1), a
    notification will be sent out to contacts indicating that the
    current host problem has been acknowledged.  If the "persistent"
    option is set to one (1), the comment associated with the
    acknowledgement will survive across restarts of the Nagios
    process.  If not, the comment will be deleted the next time
    Nagios restarts.
    """
    return send_command("ACKNOWLEDGE_HOST_PROBLEM",
                        command_file,
                        timestamp,
                        host_name,
                        sticky,
                        notify,
                        persistent,
                        author,
                        comment)


def del_svc_comment(
    comment_id,
    command_file=None,
    timestamp=0
):
    """
    Deletes a service comment.  The id number of the comment that is
    to be deleted must be specified.
    """
    return send_command("DEL_SVC_COMMENT",
                        command_file,
                        timestamp,
                        comment_id)


def acknowledge_svc_problem(
    host_name,
    service_description,
    sticky,
    notify,
    persistent,
    author,
    comment,
    command_file=None,
    timestamp=0
):
    """
    Allows you to acknowledge the current problem for the specified
    service.  By acknowledging the current problem, future
    notifications (for the same servicestate) are disabled.  If the
    "sticky" option is set to two (2), the acknowledgement will
    remain until the service returns to an OK state.  Otherwise the
    acknowledgement will automatically be removed when the service
    changes state.  If the "notify" option is set to one (1), a
    notification will be sent out to contacts indicating that the
    current service problem has been acknowledged.  If the
    "persistent" option is set to one (1), the comment associated
    with the acknowledgement will survive across restarts of the
    Nagios process.  If not, the comment will be deleted the next
    time Nagios restarts.
    """
    return send_command("ACKNOWLEDGE_SVC_PROBLEM",
                        command_file,
                        timestamp,
                        host_name,
                        service_description,
                        sticky,
                        notify,
                        persistent,
                        author,
                        comment)


def start_executing_svc_checks(
    command_file=None,
    timestamp=0
):
    """
    Enables active checks of services on a program-wide basis.
    """
    return send_command("START_EXECUTING_SVC_CHECKS",
                        command_file,
                        timestamp,
                        )


def stop_executing_svc_checks(
    command_file=None,
    timestamp=0
):
    """
    Disables active checks of services on a program-wide basis.
    """
    return send_command("STOP_EXECUTING_SVC_CHECKS",
                        command_file,
                        timestamp,
                        )


def start_accepting_passive_svc_checks(
    command_file=None,
    timestamp=0
):
    """
    Enables passive service checks on a program-wide basis.
    """
    return send_command("START_ACCEPTING_PASSIVE_SVC_CHECKS",
                        command_file,
                        timestamp,
                        )


def stop_accepting_passive_svc_checks(
    command_file=None,
    timestamp=0
):
    """
    Disables passive service checks on a program-wide basis.
    """
    return send_command("STOP_ACCEPTING_PASSIVE_SVC_CHECKS",
                        command_file,
                        timestamp,
                        )


def enable_passive_svc_checks(
    host_name,
    service_description,
    command_file=None,
    timestamp=0
):
    """
    Enables passive checks for the specified service.
    """
    return send_command("ENABLE_PASSIVE_SVC_CHECKS",
                        command_file,
                        timestamp,
                        host_name,
                        service_description)


def disable_passive_svc_checks(
    host_name,
    service_description,
    command_file=None,
    timestamp=0
):
    """
    Disables passive checks for the specified service.
    """
    return send_command("DISABLE_PASSIVE_SVC_CHECKS",
                        command_file,
                        timestamp,
                        host_name,
                        service_description)


def enable_event_handlers(
    command_file=None,
    timestamp=0
):
    """
    Enables host and service event handlers on a program-wide basis.
    """
    return send_command("ENABLE_EVENT_HANDLERS",
                        command_file,
                        timestamp,
                        )


def disable_event_handlers(
    command_file=None,
    timestamp=0
):
    """
    Disables host and service event handlers on a program-wide
    basis.
    """
    return send_command("DISABLE_EVENT_HANDLERS",
                        command_file,
                        timestamp,
                        )


def enable_host_event_handler(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Enables the event handler for the specified host.
    """
    return send_command("ENABLE_HOST_EVENT_HANDLER",
                        command_file,
                        timestamp,
                        host_name)


def enable_svc_check(
    host_name,
    service_description,
    command_file=None,
    timestamp=0
):
    """
    Enables active checks for a particular service.
    """
    return send_command("ENABLE_SVC_CHECK",
                        command_file,
                        timestamp,
                        host_name,
                        service_description)


def disable_host_event_handler(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Disables the event handler for the specified host.
    """
    return send_command("DISABLE_HOST_EVENT_HANDLER",
                        command_file,
                        timestamp,
                        host_name)


def enable_svc_event_handler(
    host_name,
    service_description,
    command_file=None,
    timestamp=0
):
    """
    Enables the event handler for the specified service.
    """
    return send_command("ENABLE_SVC_EVENT_HANDLER",
                        command_file,
                        timestamp,
                        host_name,
                        service_description)


def disable_svc_event_handler(
    host_name,
    service_description,
    command_file=None,
    timestamp=0
):
    """
    Disables the event handler for the specified service.
    """
    return send_command("DISABLE_SVC_EVENT_HANDLER",
                        command_file,
                        timestamp,
                        host_name,
                        service_description)


def enable_host_check(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Enables (regularly scheduled and on-demand) active checks of the
    specified host.
    """
    return send_command("ENABLE_HOST_CHECK",
                        command_file,
                        timestamp,
                        host_name)


def disable_host_check(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Disables (regularly scheduled and on-demand) active checks of
    the specified host.
    """
    return send_command("DISABLE_HOST_CHECK",
                        command_file,
                        timestamp,
                        host_name)


def start_obsessing_over_svc_checks(
    command_file=None,
    timestamp=0
):
    """
    Enables processing of service checks via the OCSP command on a
    program-wide basis.
    """
    return send_command("START_OBSESSING_OVER_SVC_CHECKS",
                        command_file,
                        timestamp,
                        )


def stop_obsessing_over_svc_checks(
    command_file=None,
    timestamp=0
):
    """
    Disables processing of service checks via the OCSP command on a
    program-wide basis.
    """
    return send_command("STOP_OBSESSING_OVER_SVC_CHECKS",
                        command_file,
                        timestamp,
                        )


def start_obsessing_over_host_checks(
    command_file=None,
    timestamp=0
):
    """
    Enables processing of host checks via the OCHP command on a
    program-wide basis.
    """
    return send_command("START_OBSESSING_OVER_HOST_CHECKS",
                        command_file,
                        timestamp,
                        )


def stop_obsessing_over_host_checks(
    command_file=None,
    timestamp=0
):
    """
    Disables processing of host checks via the OCHP command on a
    program-wide basis.
    """
    return send_command("STOP_OBSESSING_OVER_HOST_CHECKS",
                        command_file,
                        timestamp,
                        )


def start_obsessing_over_host(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Enables processing of host checks via the OCHP command for the
    specified host.
    """
    return send_command("START_OBSESSING_OVER_HOST",
                        command_file,
                        timestamp,
                        host_name)


def disable_svc_check(
    host_name,
    service_description,
    command_file=None,
    timestamp=0
):
    """
    Disables active checks for a particular service.
    """
    return send_command("DISABLE_SVC_CHECK",
                        command_file,
                        timestamp,
                        host_name,
                        service_description)


def stop_obsessing_over_host(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Disables processing of host checks via the OCHP command for the
    specified host.
    """
    return send_command("STOP_OBSESSING_OVER_HOST",
                        command_file,
                        timestamp,
                        host_name)


def start_obsessing_over_svc(
    host_name,
    service_description,
    command_file=None,
    timestamp=0
):
    """
    Enables processing of service checks via the OCSP command for
    the specified service.
    """
    return send_command("START_OBSESSING_OVER_SVC",
                        command_file,
                        timestamp,
                        host_name,
                        service_description)


def stop_obsessing_over_svc(
    host_name,
    service_description,
    command_file=None,
    timestamp=0
):
    """
    Disables processing of service checks via the OCSP command for
    the specified service.
    """
    return send_command("STOP_OBSESSING_OVER_SVC",
                        command_file,
                        timestamp,
                        host_name,
                        service_description)


def enable_failure_prediction(
    command_file=None,
    timestamp=0
):
    """
    Enables failure prediction on a program-wide basis.  This
    feature is not currently implemented in Nagios.
    """
    return send_command("ENABLE_FAILURE_PREDICTION",
                        command_file,
                        timestamp,
                        )


def disable_failure_prediction(
    command_file=None,
    timestamp=0
):
    """
    Disables failure prediction on a program-wide basis.  This
    feature is not currently implemented in Nagios.
    """
    return send_command("DISABLE_FAILURE_PREDICTION",
                        command_file,
                        timestamp,
                        )


def enable_performance_data(
    command_file=None,
    timestamp=0
):
    """
    Enables the processing of host and service performance data on a
    program-wide basis.
    """
    return send_command("ENABLE_PERFORMANCE_DATA",
                        command_file,
                        timestamp,
                        )


def disable_performance_data(
    command_file=None,
    timestamp=0
):
    """
    Disables the processing of host and service performance data on
    a program-wide basis.
    """
    return send_command("DISABLE_PERFORMANCE_DATA",
                        command_file,
                        timestamp,
                        )


def start_executing_host_checks(
    command_file=None,
    timestamp=0
):
    """
    Enables active host checks on a program-wide basis.
    """
    return send_command("START_EXECUTING_HOST_CHECKS",
                        command_file,
                        timestamp,
                        )


def stop_executing_host_checks(
    command_file=None,
    timestamp=0
):
    """
    Disables active host checks on a program-wide basis.
    """
    return send_command("STOP_EXECUTING_HOST_CHECKS",
                        command_file,
                        timestamp,
                        )


def start_accepting_passive_host_checks(
    command_file=None,
    timestamp=0
):
    """
    Enables acceptance and processing of passive host checks on a
    program-wide basis.
    """
    return send_command("START_ACCEPTING_PASSIVE_HOST_CHECKS",
                        command_file,
                        timestamp,
                        )


def disable_notifications(
    command_file=None,
    timestamp=0
):
    """
    Disables host and service notifications on a program-wide basis.
    """
    return send_command("DISABLE_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        )


def stop_accepting_passive_host_checks(
    command_file=None,
    timestamp=0
):
    """
    Disables acceptance and processing of passive host checks on a
    program-wide basis.
    """
    return send_command("STOP_ACCEPTING_PASSIVE_HOST_CHECKS",
                        command_file,
                        timestamp,
                        )


def enable_passive_host_checks(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Enables acceptance and processing of passive host checks for the
    specified host.
    """
    return send_command("ENABLE_PASSIVE_HOST_CHECKS",
                        command_file,
                        timestamp,
                        host_name)


def disable_passive_host_checks(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Disables acceptance and processing of passive host checks for
    the specified host.
    """
    return send_command("DISABLE_PASSIVE_HOST_CHECKS",
                        command_file,
                        timestamp,
                        host_name)


def enable_flap_detection(
    command_file=None,
    timestamp=0
):
    """
    Enables host and service flap detection on a program-wide basis.
    """
    return send_command("ENABLE_FLAP_DETECTION",
                        command_file,
                        timestamp,
                        )


def disable_flap_detection(
    command_file=None,
    timestamp=0
):
    """
    Disables host and service flap detection on a program-wide
    basis.
    """
    return send_command("DISABLE_FLAP_DETECTION",
                        command_file,
                        timestamp,
                        )


def enable_host_flap_detection(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Enables flap detection for the specified host.  In order for the
    flap detection algorithms to be run for the host, flap detection
    must be enabled on a program-wide basis as well.
    """
    return send_command("ENABLE_HOST_FLAP_DETECTION",
                        command_file,
                        timestamp,
                        host_name)


def enable_svc_flap_detection(
    host_name,
    service_description,
    command_file=None,
    timestamp=0
):
    """
    Enables flap detection for the specified service.  In order for
    the flap detection algorithms to be run for the service, flap
    detection must be enabled on a program-wide basis as well.
    """
    return send_command("ENABLE_SVC_FLAP_DETECTION",
                        command_file,
                        timestamp,
                        host_name,
                        service_description)


def disable_host_flap_detection(
    host_name,
    command_file=None,
    timestamp=0
):
    """
    Disables flap detection for the specified host.
    """
    return send_command("DISABLE_HOST_FLAP_DETECTION",
                        command_file,
                        timestamp,
                        host_name)


def disable_service_flap_detection(
    host_name,
    service_description,
    command_file=None,
    timestamp=0
):
    """
    Disables flap detection for the specified service.
    """
    return send_command("DISABLE_SERVICE_FLAP_DETECTION",
                        command_file,
                        timestamp,
                        host_name,
                        service_description)


def enable_hostgroup_svc_notifications(
    hostgroup_name,
    command_file=None,
    timestamp=0
):
    """
    Enables notifications for all services that are associated with
    hosts in a particular hostgroup.  This does not enable
    notifications for the hosts in the hostgroup - see the
    ENABLE_HOSTGROUP_HOST_NOTIFICATIONS command for that.  In order
    for notifications to be sent out for these services,
    notifications must be enabled on a program-wide basis as well.
    """
    return send_command("ENABLE_HOSTGROUP_SVC_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        hostgroup_name)


def enable_notifications(
    command_file=None,
    timestamp=0
):
    """
    Enables host and service notifications on a program-wide basis.
    """
    return send_command("ENABLE_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        )


def disable_hostgroup_svc_notifications(
    hostgroup_name,
    command_file=None,
    timestamp=0
):
    """
    Disables notifications for all services associated with hosts in
    a particular hostgroup.  This does not disable notifications for
    the hosts in the hostgroup - see the
    DISABLE_HOSTGROUP_HOST_NOTIFICATIONS command for that.
    """
    return send_command("DISABLE_HOSTGROUP_SVC_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        hostgroup_name)


def enable_hostgroup_host_notifications(
    hostgroup_name,
    command_file=None,
    timestamp=0
):
    """
    Enables notifications for all hosts in a particular hostgroup.
    This does not enable notifications for the services associated
    with the hosts in the hostgroup - see the
    ENABLE_HOSTGROUP_SVC_NOTIFICATIONS command for that.  In order
    for notifications to be sent out for these hosts, notifications
    must be enabled on a program-wide basis as well.
    """
    return send_command("ENABLE_HOSTGROUP_HOST_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        hostgroup_name)


def disable_hostgroup_host_notifications(
    hostgroup_name,
    command_file=None,
    timestamp=0
):
    """
    Disables notifications for all hosts in a particular hostgroup.
    This does not disable notifications for the services associated
    with the hosts in the hostgroup - see the
    DISABLE_HOSTGROUP_SVC_NOTIFICATIONS command for that.
    """
    return send_command("DISABLE_HOSTGROUP_HOST_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        hostgroup_name)


def enable_hostgroup_svc_checks(
    hostgroup_name,
    command_file=None,
    timestamp=0
):
    """
    Enables active checks for all services associated with hosts in
    a particular hostgroup.
    """
    return send_command("ENABLE_HOSTGROUP_SVC_CHECKS",
                        command_file,
                        timestamp,
                        hostgroup_name)


def disable_hostgroup_svc_checks(
    hostgroup_name,
    command_file=None,
    timestamp=0
):
    """
    Disables active checks for all services associated with hosts in
    a particular hostgroup.
    """
    return send_command("DISABLE_HOSTGROUP_SVC_CHECKS",
                        command_file,
                        timestamp,
                        hostgroup_name)


def enable_hostgroup_host_checks(
    hostgroup_name,
    command_file=None,
    timestamp=0
):
    """
    Enables active checks for all hosts in a particular hostgroup.
    """
    return send_command("ENABLE_HOSTGROUP_HOST_CHECKS",
                        command_file,
                        timestamp,
                        hostgroup_name)


def disable_hostgroup_host_checks(
    hostgroup_name,
    command_file=None,
    timestamp=0
):
    """
    Disables active checks for all hosts in a particular hostgroup.
    """
    return send_command("DISABLE_HOSTGROUP_HOST_CHECKS",
                        command_file,
                        timestamp,
                        hostgroup_name)


def enable_hostgroup_passive_host_checks(
    hostgroup_name,
    command_file=None,
    timestamp=0
):
    """
    Enables passive checks for all hosts in a particular hostgroup.
    """
    return send_command("ENABLE_HOSTGROUP_PASSIVE_HOST_CHECKS",
                        command_file,
                        timestamp,
                        hostgroup_name)


def disable_hostgroup_passive_host_checks(
    hostgroup_name,
    command_file=None,
    timestamp=0
):
    """
    Disables passive checks for all hosts in a particular hostgroup.
    """
    return send_command("DISABLE_HOSTGROUP_PASSIVE_HOST_CHECKS",
                        command_file,
                        timestamp,
                        hostgroup_name)


def enable_hostgroup_passive_svc_checks(
    hostgroup_name,
    command_file=None,
    timestamp=0
):
    """
    Enables passive checks for all services associated with hosts in
    a particular hostgroup.
    """
    return send_command("ENABLE_HOSTGROUP_PASSIVE_SVC_CHECKS",
                        command_file,
                        timestamp,
                        hostgroup_name)


def restart_program(
    command_file=None,
    timestamp=0
):
    """
    Restarts the Nagios process.
    """
    return send_command("RESTART_PROGRAM",
                        command_file,
                        timestamp,
                        )


def disable_hostgroup_passive_svc_checks(
    hostgroup_name,
    command_file=None,
    timestamp=0
):
    """
    Disables passive checks for all services associated with hosts
    in a particular hostgroup.
    """
    return send_command("DISABLE_HOSTGROUP_PASSIVE_SVC_CHECKS",
                        command_file,
                        timestamp,
                        hostgroup_name)


def enable_servicegroup_svc_notifications(
    servicegroup_name,
    command_file=None,
    timestamp=0
):
    """
    Enables notifications for all services that are members of a
    particular servicegroup.  In order for notifications to be sent
    out for these services, notifications must also be enabled on a
    program-wide basis.
    """
    return send_command("ENABLE_SERVICEGROUP_SVC_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        servicegroup_name)


def disable_servicegroup_svc_notifications(
    servicegroup_name,
    command_file=None,
    timestamp=0
):
    """
    Disables notifications for all services that are members of a
    particular servicegroup.
    """
    return send_command("DISABLE_SERVICEGROUP_SVC_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        servicegroup_name)


def enable_servicegroup_host_notifications(
    servicegroup_name,
    command_file=None,
    timestamp=0
):
    """
    Enables notifications for all hosts that have services that are
    members of a particular servicegroup.  In order for
    notifications to be sent out for these hosts, notifications must
    also be enabled on a program-wide basis.
    """
    return send_command("ENABLE_SERVICEGROUP_HOST_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        servicegroup_name)


def disable_servicegroup_host_notifications(
    servicegroup_name,
    command_file=None,
    timestamp=0
):
    """
    Disables notifications for all hosts that have services that are
    members of a particular servicegroup.
    """
    return send_command("DISABLE_SERVICEGROUP_HOST_NOTIFICATIONS",
                        command_file,
                        timestamp,
                        servicegroup_name)


def enable_servicegroup_svc_checks(
    servicegroup_name,
    command_file=None,
    timestamp=0
):
    """
    Enables active checks for all services in a particular
    servicegroup.
    """
    return send_command("ENABLE_SERVICEGROUP_SVC_CHECKS",
                        command_file,
                        timestamp,
                        servicegroup_name)


def disable_servicegroup_svc_checks(
    servicegroup_name,
    command_file=None,
    timestamp=0
):
    """
    Disables active checks for all services in a particular
    servicegroup.
    """
    return send_command("DISABLE_SERVICEGROUP_SVC_CHECKS",
                        command_file,
                        timestamp,
                        servicegroup_name)


def enable_servicegroup_host_checks(
    servicegroup_name,
    command_file=None,
    timestamp=0
):
    """
    Enables active checks for all hosts that have services that are
    members of a particular hostgroup.
    """
    return send_command("ENABLE_SERVICEGROUP_HOST_CHECKS",
                        command_file,
                        timestamp,
                        servicegroup_name)


def disable_servicegroup_host_checks(
    servicegroup_name,
    command_file=None,
    timestamp=0
):
    """
    Disables active checks for all hosts that have services that are
    members of a particular hostgroup.
    """
    return send_command("DISABLE_SERVICEGROUP_HOST_CHECKS",
                        command_file,
                        timestamp,
                        servicegroup_name)


def enable_servicegroup_passive_svc_checks(
    servicegroup_name,
    command_file=None,
    timestamp=0
):
    """
    Enables the acceptance and processing of passive checks for all
    services in a particular servicegroup.
    """
    return send_command("ENABLE_SERVICEGROUP_PASSIVE_SVC_CHECKS",
                        command_file,
                        timestamp,
                        servicegroup_name)



#!/usr/bin/env python
#
# This example shows how to work with attributes in nagios that
# Are in a comma-seperated form. Like this:
#
# define service {
#    contact_groups      +admins,webmasters,hostmasters
#    host_name           host.example.com
#    service_description Test Service
# }
#

import pynag.Model

# First create a test object
my_service = pynag.Model.Service()
my_service['host_name'] = 'examplehost'
my_service['service_description'] = 'Test Service'
my_service['contact_groups'] = '+admins,webmasters,hostmasters'

print("*** Created a demo servicecheck that looks like this:")
print(my_service)



print("\n--- Removing with attribute_removefield()")
my_service.attribute_removefield('contact_groups', 'hostmasters')
print("my_service.contact_groups = ", my_service.contact_groups)



print("\n--- Add a new contact_group with attribute_appendfield()")
my_service.attribute_appendfield('contact_groups', "mycontactgroup")
print("my_service.contact_groups = ", my_service.contact_groups)

print("\n-- Replacing a contact_group midfield with attribute_replacefield()")
my_service.attribute_replacefield('contact_groups', "webmasters", "hostmaster")
print("my_service.contact_groups = ", my_service.contact_groups)


# A more advanced example. Find all services that inherit from "generic-service" and
# Replace it with "my-specific-service":

print("\n--- More advanced example, editing multiple objects at once...")
my_services = pynag.Model.Service.objects.filter(use__hasfield='generic-service')

for service in my_services:
    service.attribute_replacefield('use','generic-service','my-specific-service')
    # service.save()





#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" General Utilities from importing nagios objects. Currently .csv files are supported

Either execute this script standalone from the command line or use it as a python library like so:

>>> from pynag.Utils import importer
>>> pynag_objects = importer.import_from_csv_file(filename='foo', seperator=',') # doctest: +SKIP
>>> for i in pynag_objects: # doctest: +SKIP
...     i.save() # doctest: +SKIP
"""

import optparse
import os

import pynag.Model

# Default host is used when you try to add a service to a
# Host that does not exist
DEFAULT_HOST_ATTRIBUTES = {
    'use': 'generic-host',
    'check_command': 'check-host-alive',
    'check_interval': '1',
    'max_check_attempts': '3',
    'retry_interval': '2',
}


options = optparse.OptionParser()
options.add_option(
    '--destination_filename',
    dest='destination_filename',
    default=None,
    help='Destination file where objects will be saved',
)
options.add_option(
    '--seperator',
    dest='seperator',
    default=',',
    help='Use this as seperator for columns (default: ,)',
)
options.add_option(
    '--dry_run',
    dest='dry_run',
    default=False,
    action='store_true',
    help='If specified, only print object definitions to screen',
)
options.add_option(
    '--object_type',
    dest='object_type',
    default=None,
    help='Assume this object type (e.g. "host") if its not specified in file'
)


def parse_arguments():
    """ Parse command line arguments """
    (opts, args) = options.parse_args()
    return opts, args


def main():
    (opts, args) = parse_arguments()

    if not args:
        options.error("You must specify at least one .csv file as an argument")
    for i in args:
        if not os.path.isfile(i):
            options.error("Could not find file: %s" % i)
    pynag_objects = []
    for i in args:
        tmp = import_from_csv_file(filename=i, seperator=opts.seperator, object_type=opts.object_type)
        pynag_objects += tmp

    if opts.dry_run:
        print(len(pynag_objects))
        for i in pynag_objects:
            print(i)
    else:
        for i in pynag_objects:
            i.save(filename=opts.destination_filename)


def dict_to_pynag_objects(dict_list, object_type=None):
    """Take a list of dictionaries, return a list of pynag.Model objects.

    Args:
        dict_list: List of dictionaries that represent pynag objects
        object_type: Use this object type as default, if it is not specified in dict_list
    Returns:
        List of pynag objects
    """
    result = []
    for i in dict_list:
        dictionary = i.copy()
        object_type = dictionary.pop("object_type", object_type)
        if not object_type:
            raise ValueError('One column needs to specify object type')
        Class = pynag.Model.string_to_class[object_type]
        pynag_object = Class(**dictionary)
        result.append(pynag_object)
    return result


def parse_csv_file(filename, seperator=','):
    """ Parse filename and return a dict representing its contents """
    with open(filename) as f:
        data = f.read()
        return parse_csv_string(data, seperator=seperator)


def parse_csv_string(csv_string, seperator=','):
    """ Parse csv string and return a dict representing its contents """
    result = []
    lines = csv_string.splitlines()
    headers = None
    for line in lines:
        # Skip empty lines
        if not line:
            continue
        # If this is first line
        if not headers:
            headers = [x.strip() for x in line.split(seperator)]
            continue
        mydict = {}
        result.append(mydict)
        columns = [x.strip() for x in line.split(seperator)]
        for i, column in enumerate(columns):
            header = headers[i]
            mydict[header] = column
    return result


def import_from_csv_file(filename, seperator=',', object_type=None):
    """ Parses filename and returns a list of pynag objects.

    Args:
        filename: Path to a file
        seperator: use this symbol to seperate columns in the file
        object_type: Assume this object_type if there is no object_type column
    """
    dicts = parse_csv_file(filename=filename, seperator=seperator)
    pynag_objects = dict_to_pynag_objects(dicts, object_type=object_type)
    return pynag_objects

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""Module for errors shared among pynag.Parsers package."""

import pynag.errors


class ParserError(pynag.errors.PynagError):
    """ ParserError is used for errors that the Parser has when parsing config.

    Typical usecase when there is a critical error while trying to read configuration.
    """
    filename = None
    line_start = None
    message = None

    def __init__(self, message=None, item=None):
        """ Creates an instance of ParserError

        Args:

            message: Message to be printed by the error

            item: Pynag item who caused the error

        """
        self.message = message
        if item is None:
            return
        self.item = item
        self.filename = item['meta']['filename']
        self.line_start = item['meta'].get('line_start')

    def __str__(self):
        message = self.message
        if self.filename and self.line_start:
            message = '%s in %s, line %s' % (message, self.filename, self.line_start)
        return repr(message)



#!/usr/bin/env python
#
# Example script on how to modify a single Nagios service

from pynag import Model
import sys

# Parse commandline arguments
if len(sys.argv) != 5:
    print('''
Usage:
  %s <host_name> <service_description> <field_name> <new_value>

Example:
  %s localhost Ping check_command 'check_ping'
''' % (sys.argv[0], sys.argv[0]))
    sys.exit(1)

host_name = sys.argv[1]
service_description = sys.argv[2]
field_name = sys.argv[3]
new_value = sys.argv[4]


# Get a list of all services that fit our search pattern
search_results = Model.Service.objects.filter(host_name=host_name, service_description=service_description)

if len(search_results) == 0:
    print("no service found for host_name=%s and service_description=%s" % ( host_name, service_description ))

my_service = search_results[0]
my_service.set_attribute(field_name,new_value)
my_service.save()





# -*- coding: utf-8 -*-
"""Module for parsing and filtering object_cache files."""

from pynag.Parsers import config_parser


class ObjectCache(config_parser.Config):

    """ Loads the configuration as it appears in objects.cache file """

    def get_cfg_files(self):
        for k, v in self.maincfg_values:
            if k == 'object_cache_file':
                return [v]
